#include "R3Graph.h"
#include "tetrahed.h"

namespace R3Graph {

double Tetrahedron::innerRadius() const {
    double v = volume();
    double s =
        (vertex[1] - vertex[0]).area(vertex[2] - vertex[0]) +
        (vertex[1] - vertex[0]).area(vertex[3] - vertex[0]) +
        (vertex[2] - vertex[0]).area(vertex[3] - vertex[0]) +
        (vertex[2] - vertex[1]).area(vertex[3] - vertex[1]);
    s *= 0.5;
    if (s <= R3_EPSILON)
        return 0.;
    return (3.*v/s);
}

double Tetrahedron::outerRadius() const {
    R3Vector n0 = vertex[1] - vertex[0];
    R3Point p0 = vertex[0] + n0 * 0.5;
    n0.normalize();

    R3Vector n1 = vertex[2] - vertex[0];
    R3Point p1 = vertex[0] + n1 * 0.5;
    n1.normalize();

    R3Vector n2 = vertex[3] - vertex[0];
    R3Point p2 = vertex[0] + n2 * 0.5;
    n2.normalize();

    R3Point p;
    R3Vector v;

    if (!intersectPlanes(
        p0, n0,
        p1, n1,
        p, v
    ))
        return 1.e+30;

    R3Point center;

    if (!intersectPlaneAndLine(
        p2, n2,
        p, v,
        center
    ))
        return 1.e+30;

    return center.distance(vertex[0]);
}

double Tetrahedron::aspect() const {
    double r = innerRadius();
    if (r <= R3_EPSILON)
        return 0.;
    else
        return outerRadius() / r;
}

} // end of namespace R3Graph
